// -*-Mode:C++-*-

/* Open SoundControl kit in C++                                              */
/* Copyright (C) 2002 Daniel Holth, Clinton McChesney                        */
/*                                                                           */
/* This library is free software; you can redistribute it and/or             */
/* modify it under the terms of the GNU Lesser General Public                */
/* License as published by the Free Software Foundation; either              */
/* version 2.1 of the License, or (at your option) any later version.        */
/*                                                                           */
/* This library is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         */
/* Lesser General Public License for more details.                           */
/*                                                                           */
/* You should have received a copy of the GNU Lesser General Public          */
/* License along with this library; if not, write to the Free Software       */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
/*                                                                           */
/* For questions regarding this program contact                              */
/* Daniel Holth <dholth@stetson.edu> or visit                                */
/* http://www.stetson.edu/~ProctoLogic/                                      */

/**
 * @class Broadcaster
 *
 * Sends a message to n subscribed hosts.  (Where n is 0 or 1)
 *
 * @author Daniel Holth
 * @author Clinton McChesney
 * @author Martin Rumori
 */

#ifndef BROADCASTER_H
#define BROADCASTER_H

#include "Transmit.h"
#include <iostream>
#include <string>
#include <list>
#include <utility>
#include <time.h>

#define LEASE_LENGTH 30000

class Broadcaster : public Transmit 
{
  protected:
    typedef std::pair<time_t, Transmit*>   RankedTransmit;
    typedef std::list< RankedTransmit > TransmitList;
  
  private:
    TransmitList mySubscribers;

  protected:
    virtual bool transmitInternal(Transmit* destination, const std::string *data);
    
  public:
    Broadcaster()   {  }

    void addSubscriber(Transmit *subscriber);

    void removeSubscriber(Transmit *subscriber);
    
    virtual bool transmit(const std::string &data);

    TransmitList* getSubscribers()  { return &this->mySubscribers; }
    time_t getLeaseLength()         { return LEASE_LENGTH; }

	/// Not implemented.
	virtual Broadcaster *clone() { return NULL; }

	virtual std::string getName();
};

#endif
